local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local GetPowerRegen = GetPowerRegen
local GetTime = GetTime
local math = math
local pairs = pairs

function a.FocusAdded()
	local added = 9
	if c.WearingSet(2, "T13") then
		added = added + 9
	end
	if s.HealthPercent() <= 25 then
		added = added + 3 * c.GetTalentRank("Termination")
	end
	return added
end

local function adjustCost(info)
	if c.InfoMatches(info, "Cobra Shot", "Steady Shot") then
		info.Cost = -a.FocusAdded()
	end
end

a.Focus = 0
a.Rotations = {}
c.RegisterForEvents(a)
s.Spam[AddonName] = function()
	c.Init(a)
	local _, regen = GetPowerRegen()
	adjustCost(c.GetCastingInfo())
	adjustCost(c.GetQueuedInfo())
	a.Focus = c.GetPower(regen)
	c.Flash(a)
end

----------------------------------------------------------------- Beast Mastery
a.Rotations.BeastMastery = {
	Spec = 1,
	OffSwitch = "bm_off",
	
	FlashInCombat = function()
		c.FlashAll(
			"Hunter's Mark", 
			"Focus Fire",
			"Fervor")
		if c.AoE then
			c.FlashAll(
				"Bestial Wrath for AoE",
				"Burrow Attack",
				"Froststorm Breath")
			c.PriorityFlash(
				"Explosive Trap",
				"Multi-Shot for BM", 
				"Kill Shot", 
				"Cobra Shot")
		else
--c.Debug(a.Focus, s.Power("player"),
			c.PriorityFlash(
				"Bestial Wrath",
				"Kill Command",
				"Serpent Sting", 
				"Kill Shot",
				"Arcane Shot over 64",
				"Cobra Shot")
--, GetTime() + c.GetBusyTime())
		end
	end,
	
	FlashAlways = function()
		c.FlashAll("Aspect of the Hawk", "Mend Pet", "Call Pet")
	end,
}

------------------------------------------------------------------ Marksmanship
local nextSSIsImproved
local ssUnimprovers = {
	"Aimed Shot",
	"Aimed Shot!",
	"Chimera Shot",
	"Serpent Sting",
	"Arcane Shot",
	"Multi-Shot",
	"Distracting Shot",
	"Tranquilizing Shot",
}
-- Things that do NOT break ISS:
-- Traps
-- Things not on the GCD
-- Deterrence
-- Disengage
-- Feign Death
-- Flare
-- Hunter's Mark
-- Mend Pet
-- Readiness

local function getImprovedStatus(info, curValue)
	if info then
		if c.InfoMatches(info, "Steady Shot") then
			return not curValue
		elseif c.InfoMatches(info, ssUnimprovers) then
			return false
		end
	end
	return curValue
end

a.Rotations.Marksmanship = {
	Spec = 2,
	OffSwitch = "marks_off",
	
	FlashInCombat = function()
		
		-- adjust resources/cooldowns to reflect casting & queued spells
		local casting = c.GetCastingInfo()
		local queued = c.GetQueuedInfo()
		a.NextSSIsImproved = getImprovedStatus(casting, nextSSIsImproved)
		a.NextSSIsImproved = getImprovedStatus(queued, a.NextSSIsImproved)
		if c.InfoMatches(queued, "Chimera Shot") then
			a.CSCool = 10
		else
			a.CSCool = c.GetCooldown("Chimera Shot")
		end
		if not a.NextSSIsImproved
			and (c.InfoMatches(casting, "Steady Shot")
				or c.InfoMatches(queued, "Steady Shot")) then
			
			a.ISSDur = 8
		else
			a.ISSDur = c.GetBuffDuration("Improved Steady Shot")
		end
		
		-- flash
		if c.AoE then
			c.FlashAll("Kiroptyric Sigil", "Hunter's Mark")
			c.PriorityFlash(
				"Steady Shot 2 AoE",
				"Explosive Trap",
				"Multi-Shot",
				"Aimed Shot!",
				"Kill Shot",
				"Steady Shot")
		else
			c.FlashAll("Kiroptyric Sigil")
			c.PriorityFlash(
				"Steady Shot 2",
				"Serpent Sting unless Careful Aim",
				"Chimera Shot",
				"Aimed Shot!",
				"Steady Shot No Buff",
				"Kill Shot",
				"Rapid Fire if Talented",
				"Readiness",
				"Steady Shot Refresh Buff",
				"Marksmanship Focus Dump",
				"Kill Command under Resistance is Futile",
				"Steady Shot")
		end
	end,
	
	FlashAlways = function()
		c.FlashAll("Aspect of the Hawk", "Mend Pet", "Call Pet")
	end,
	
	CastSucceeded = function(info)
		nextSSIsImproved = getImprovedStatus(info, nextSSIsImproved)
	end,
}

---------------------------------------------------------------------- Survival
local secondESFired = false
local esFireTime = 0

a.Rotations.Survival = {
	Spec = 3,
	OffSwitch = "survival_off",
	
	FlashInCombat = function()
		local esIdle = GetTime() + c.GetBusyTime() - esFireTime
		if esIdle > 2.5 then
			a.AllowES = true
		elseif secondESFired then
			a.AllowES = esIdle > 1.5
		else
			a.AllowES = not c.InfoMatches(c.GetQueuedInfo(), "Explosive Shot")
		end
		
		if c.AoE then
			c.FlashAll("Hunter's Mark", "Kiroptyric Sigil")
			c.PriorityFlash(
				"Explosive Trap",
				"Multi-Shot",
				"Explosive Shot for Free",
				"Kill Shot",
				"Cobra Shot")
		else
			c.FlashAll(
				"Hunter's Mark", 
				"Serpent Sting", 
				"Black Arrow", 
				"Kiroptyric Sigil")
			c.PriorityFlash(
				"Explosive Shot",
				"Kill Shot",
				"Arcane Shot over 64",
				"Cobra Shot")
		end
	end,
	
	FlashAlways = function()
		c.FlashAll("Aspect of the Hawk", "Mend Pet", "Call Pet")
	end,
	
	CastSucceeded = function(info)
		if c.InfoMatches(info, "Explosive Shot") then
			if GetTime() - esFireTime < 2 then
				secondESFired = true
			else
				secondESFired = false
			end
			esFireTime = GetTime()
			c.Debug("Event", "Second Explosive Shot:", secondESFired)
		end
	end,
}
